/**
 * ================================
 * Forms BI Bean Graph Integration
 * ================================
 * 
 * @author   Frank Nimphius
 * @version  1.0
 * 
 * history of changes 
 * ================================
 * 1. November 2002 initial creation
*/

package oracle.forms.demos.bigraph;
import oracle.dss.graph.Graph;
import java.lang.Exception;
/**
 * graphTypeRegistry maps BI Graph types to the named strings passed as an argument to
 * the getTypeForString method. The following Graph type names can be placed as an argument
 * 
 * The main use of this registry is to limit the number of supported Graphs in Forms. To add 
 * additional graph types, just add the graph type definition to this class
 */
public abstract class graphTypeRegistry 
{
  public graphTypeRegistry()
  {
  }

  /**
   * getTypeForString(String gt) retrieves the dss Graph internal number for the Graph type specified in the string. 
   * If no matching Graph type is found then the method returns 0
   */
  public static int getTypeForString(String gt) throws Exception
  {
      int graphTypeReturn = 9999999;
      
       if ("HORIZONTAL_BAR".equalsIgnoreCase(gt))
       {
         graphTypeReturn = Graph.BAR_HORIZ_CLUST;
       }
       else if("HORIZONTAL_BAR_2Y".equalsIgnoreCase(gt))
       {
         graphTypeReturn = Graph.BAR_HORIZ_CLUST_2Y;
       }
       else if ("VERTICAL_BAR".equalsIgnoreCase(gt))
       {
         graphTypeReturn = Graph.BAR_VERT_CLUST;
       }
       else if ("VERTICAL_BAR_2Y".equalsIgnoreCase(gt))
       {
         graphTypeReturn = Graph.BAR_VERT_CLUST2Y;
       }
       else if ("VERTICAL_STACKED_BAR".equalsIgnoreCase(gt))
       {
         graphTypeReturn = Graph.BAR_VERT_STACK;
       }
       else if ("HORIZONTAL_STACKED_BAR".equalsIgnoreCase(gt))
       {
         graphTypeReturn = Graph.BAR_HORIZ_STACK;
       }
       else if ("VERTICAL_PERCENT_BAR".equalsIgnoreCase(gt))
       {
         graphTypeReturn = Graph.BAR_VERT_PERCENT;
       }
       else if ("HORIZONTAL_PERCENT_BAR".equalsIgnoreCase(gt))
       {
         graphTypeReturn = Graph.BAR_HORIZ_PERCENT;
       }
       else if ("VERTICAL_LINE_GRAPH".equalsIgnoreCase(gt))
       {
         graphTypeReturn = Graph.LINE_VERT_ABS;
       }
       else if ("HORIZONTAL_LINE_GRAPH".equalsIgnoreCase(gt))
       {
         graphTypeReturn = Graph.LINE_HORIZ_ABS;
       }
        else if ("VERTICAL_LINE_SPLIT_GRAPH".equalsIgnoreCase(gt))
       {
         graphTypeReturn = Graph.LINE_VERT_ABS_SPLIT2Y ;
       }
       else if ("VERTICAL_STACKED_LINE_GRAPH".equalsIgnoreCase(gt))
       {
         graphTypeReturn = Graph.LINE_VERT_STACK;
       }
       else if ("HORIZONTAL_STACKED_LINE_GRAPH".equalsIgnoreCase(gt))
       {
         graphTypeReturn = Graph.LINE_HORIZ_STACK;
       }
       else if ("VERTICAL_AREA_GRAPH".equalsIgnoreCase(gt))
       {
         graphTypeReturn = Graph.AREA_VERT_ABS;
       }
       else if ("VERTICAL_PERCENT_AREA_GRAPH".equalsIgnoreCase(gt))
       {
         graphTypeReturn = Graph.AREA_VERT_PERCENT;
       }

       else if ("VERTICAL_STACKED_AREA_GRAPH".equalsIgnoreCase(gt))
       {
         graphTypeReturn = Graph.AREA_VERT_STACK;
       }
       else if ("PIE_GRAPH".equalsIgnoreCase(gt))
       {
         graphTypeReturn = Graph.PIE;
       }
       else if ("PIE_BAR_GRAPH".equalsIgnoreCase(gt))
       {
         graphTypeReturn = Graph.PIE_BAR;
       }
       else if ("MULTI_PIE_GRAPH".equalsIgnoreCase(gt))
       {
         graphTypeReturn = Graph.PIE_MULTI;
       }
       else if ("MULTI_RING_GRAPH".equalsIgnoreCase(gt))
       {
         graphTypeReturn = Graph.RING_MULTI;
       }
       else if ("MULTI_PIE_PROPORTIONAL_GRAPH".equalsIgnoreCase(gt))
       {
         graphTypeReturn = Graph.PIE_MULTI_PROP;
       }
       else if ("MULTI_RING_PROPORTIONAL_GRAPH".equalsIgnoreCase(gt))
       {
         graphTypeReturn = Graph.RING_MULTI_PROP;
       }
       else if ("STOCK_HIGHLOW_GRAPH".equalsIgnoreCase(gt))
       {
         graphTypeReturn = Graph.STOCK_HILO;
       }
        else if ("RADAR".equalsIgnoreCase(gt))
       {
         graphTypeReturn = Graph.RADAR_LINE;
       }
       else if ("RING_BAR_GRAPH".equalsIgnoreCase(gt))
       {
         graphTypeReturn = Graph.RING_BAR;
       }
       else if ("3D_BAR_GRAPH".equalsIgnoreCase(gt))
       {
         graphTypeReturn = Graph.THREED_BAR;
       }
       else if ("3D_AREA_GRAPH".equalsIgnoreCase(gt))
       {
         graphTypeReturn = Graph.THREED_AREA_SERIES;
       }
       else if ("3D_AREA_GROUP_GRAPH".equalsIgnoreCase(gt))
       {
         graphTypeReturn = Graph.THREED_AREA_GROUP;
       }
      else if ("3D_CUBE_GRAPH".equalsIgnoreCase(gt))
       {
         graphTypeReturn = Graph.THREED_CUBE;
       }
       return graphTypeReturn;
  }
}