/**
 * ================================
 * Forms BI Bean Graph Integration
 * ================================
 * 
 * @author   Frank Nimphius
 * @version  1.0
 * 
 * history of changes 
 * ================================
 * 1. November 2002 initial creation
*/

package oracle.forms.demos.bigraph;

import oracle.dss.dataView.ComponentHandle;
import oracle.dss.dataView.DataComponentHandle;
import oracle.dss.dataView.GroupComponentHandle;
import oracle.dss.dataView.NonDataComponentHandle;
import oracle.dss.dataView.SeriesComponentHandle;
import oracle.dss.dataView.ViewMouseEvent;
import oracle.dss.dataView.ViewMouseListener;
import oracle.dss.graph.Graph;
import oracle.dss.util.ColumnOutOfRangeException;
import oracle.dss.util.DataDirector;
import oracle.dss.util.DataMap;
import oracle.dss.util.EdgeOutOfRangeException;
import oracle.dss.util.MetadataMap;
import oracle.dss.util.RowOutOfRangeException;
import oracle.dss.util.SliceOutOfRangeException;
/**
 * mViewMouseListener implements ViewMouseListener, handling mouse events
 * in the BI Graph
 */
public class mViewMouseListener implements ViewMouseListener
{
  Graph m_graph=null;
  FormsGraph mfg = null;
  localRelationalData mlrd = null;
  
  public mViewMouseListener(Graph g, FormsGraph fg, localRelationalData lrd)
  {
    m_graph=g;
    mfg = fg;
    mlrd = lrd;

  }

  /**
   * If mouse interaction is enabled in Forms then the following values are returned in a single 
   * delimited string. 
   * <p>
   * Label of the row:                for the Emp table this would be e.g. the employees name ("Smith")
   * </p><p>
   * Label of the column:             for the Emp table this would be e.g. Sal
   * </p><p>
   * Value of the graph:              the value is returned in 00.00 format
   * </p><p>
   * Column number of the bar in the 
   * graph:                           showing e.g. "Sal" and "Commission" will return 
   *                                  1 for "Sal" and 2 for "Commission"
   * </p><p>                                 
   * Row number of the bar in the 
   * graph:                           Showing e.g. Salary for "Smith", "Jones" and "King" will return
   *                                  1 for "Smtith", 2 for "Jones" and 3 form "King"
   * </p><p>
   * The string values are separated by the actual delimiter used, the default delimiter 
   * is ",".
   * </p>
   * 
   */
   public void mouseClicked(ViewMouseEvent p0)
  { 
    int iColNum=0;
    int iRowNum=0;
    
    try {
          ComponentHandle ch = p0.getComponentHandle();
          Object graphData = null;
          String rowLabel="", columnLabel="";

          // if the graph type is of PIE_BAR then clicking on one slice 
          // drills down to the details - shown in the bar. In case of a
          // PIE_BAR the pie is not of type data component, but series c
          // component
          
          if (ch instanceof SeriesComponentHandle){
              if ((m_graph.getGraphType() == Graph.PIE_BAR) || (m_graph.getGraphType() == Graph.RING_BAR) )
                {
                  DebugMessage("MouseListener - detect Mouse Click onto Series component");
                  m_graph.setPieBarSeries(((SeriesComponentHandle)ch).getSeries());
                }
            }
          
          if (ch instanceof DataComponentHandle)
          {
              iColNum = ((DataComponentHandle)ch).getColumn();
              iRowNum = ((DataComponentHandle)ch).getRow();
            
              DebugMessage("mouseClicked(): Number of column in graph: " + (iColNum+1));
              DebugMessage("mouseClicked(): Number of row in graph: " + (iRowNum+1));

              graphData = m_graph.getGraphModel().getDataAccess().getValue(iRowNum,iColNum,DataMap.DATA_UNFORMATTED);
              DebugMessage("mouseClicked(): Data value clicked on: "+graphData.toString());
                 
              columnLabel=(String) m_graph.getDataAccessSliceLabel(DataDirector.COLUMN_EDGE,iColNum,MetadataMap.METADATA_LONGLABEL);
              DebugMessage("mouseClicked(): Column Name clicked on: "+columnLabel);

              rowLabel=(String) m_graph.getDataAccessSliceLabel(DataDirector.ROW_EDGE ,iRowNum,MetadataMap.METADATA_LONGLABEL);
              DebugMessage("mouseClicked(): Row Name clicked on: "+rowLabel);

              /*
              * ********************************
              * Dispatch graph information to 
              * Forms
              *********************************/
              
              String sDelimiter=mfg.getDelimiter();
              String GraphInfo="";

              /*
               * Determine the data returned by this mouse click
               */
              if (mfg.returnValueSelection == mfg.allData){
                  GraphInfo =rowLabel+sDelimiter+columnLabel+sDelimiter+graphData.toString();
                  DebugMessage("MouseListener - Mouse Click - Dispatch value "+GraphInfo+" to Forms");
                  mfg.dispatchMouseAction(GraphInfo);
              }
              else if (mfg.returnValueSelection == mfg.dataLabel)
              {
                DebugMessage("mouseClicked(): MouseListener: Mouse Click returns rowLabel");

                // if the group (column data) is shown as row data (series) then the rowlabels and column
                // labels needs to be swapped 
               if(mfg.showGraphAsSeries)
               {
                GraphInfo = columnLabel; 
                DebugMessage("MouseListener - Mouse Click - Dispatch value "+GraphInfo+" to Forms");
               }else{
                GraphInfo = rowLabel;
                DebugMessage("MouseListener - Mouse Click - Dispatch value "+GraphInfo+" to Forms");
               }
                mfg.dispatchMouseAction(GraphInfo);
              }
                 else if (mfg.returnValueSelection == mfg.dataColumn)
              {
                DebugMessage("mouseClicked(): MouseListener: Mouse Click returns columnLabel");

                // if the group (column data) is shown as row data (series) then the rowlabels and column
                // labels needs to be swapped 
                if (mfg.showGraphAsSeries) 
                {
                    GraphInfo = rowLabel;
                    DebugMessage("MouseListener - Mouse Click - Dispatch value "+GraphInfo+" to Forms");
              
                } else 
                {
                  GraphInfo = columnLabel;
                  DebugMessage("MouseListener - Mouse Click - Dispatch value "+GraphInfo+" to Forms");
                }
                
                mfg.dispatchMouseAction(GraphInfo);
              }
                 else if (mfg.returnValueSelection == mfg.dataValue)
              {
                DebugMessage("mouseClicked(): MouseListener: Mouse Click returns rowValue");
                GraphInfo = graphData.toString();
                DebugMessage("MouseListener - Mouse Click - Dispatch value "+GraphInfo+" to Forms");
                mfg.dispatchMouseAction(GraphInfo);
              }
              //  return primary key only
                 else if (mfg.returnValueSelection == mfg.dataPrimKey)
              {
                String dataObject=null;
                
                DebugMessage("mouseClicked(): MouseListener: Mouse Click returns Primary Key");

                // if the group (column data) is shown as row data (series) then the rowlabels and column
                // labels needs to be swapped 
                if (mfg.showGraphAsSeries)
                {
                    dataObject = rowLabel+sDelimiter+columnLabel+sDelimiter+graphData.toString();
                }
                else
                {
                    dataObject = columnLabel+sDelimiter+rowLabel+sDelimiter+graphData.toString();
                }
                GraphInfo = mlrd.getPrimaryKey(dataObject,sDelimiter);
                DebugMessage("MouseListener - Mouse Click - Dispatch value "+GraphInfo+" to Forms");
                mfg.dispatchMouseAction(GraphInfo);
              }
              else
              {
                DebugMessage("mouseClicked(): Mouse Listener: Mouse Click returns no value");
                // no value as none is selected for returnValueSelection
              }
            }
        }  
   
    catch (ColumnOutOfRangeException coore) 
    {
      coore.printStackTrace();
    }
    catch (RowOutOfRangeException roore)
    {
      roore.printStackTrace();
    }

    catch (SliceOutOfRangeException soore)
    {
      soore.printStackTrace();
    }

    catch (EdgeOutOfRangeException eoore)
    {
      eoore.printStackTrace();
    }
    }


  public void mousePressed(ViewMouseEvent p0)
  {
  }

  public void mouseReleased(ViewMouseEvent p0)
  {
  }

  public void mouseEntered(ViewMouseEvent p0)
  {
  }

  public void mouseExited(ViewMouseEvent p0)
  {
  }

  private void DebugMessage(String dm)
  {
    mfg.DebugMessage(dm);
  }
}