package oracle.forms.demos.beans;
import javax.swing.JColorChooser;
import javax.swing.UIManager;
import javax.swing.SwingUtilities;
import java.awt.Component;
import java.awt.Color;


/**
 * <H3><FONT COLOR="#002299">Pluggable Java Components for Oracle Forms</FONT></H3>
 * <P>This simple JavaBean allows a Swing JColorChooser dialog to be displayed in a
 * Form.
 * It essentially has a single method which displays a color picker dialog and returns
 * the color to Forms as an RGB triple or null
 * </P>
 * <P>
 * All of the datatype encoding from Color to String and visa versa is handled by the
 * built-in encoders that are part of the enhanced JavaBean support introduced in 9<i>i</i>
 * </P>
 * <P><u>Change History</u></P>
 * drmills 30-Oct-2001 - Creation
 *
 * @version 9.0.0
 * @date 30-Oct-2001
 * @author Duncan Mills
 */

public class ColorPicker extends Component
{
  /**
   * the classname used for debugging purposes
   */
  private final String CLASSNAME = this.getClass().getName();

  /**
   * are we debugging?
   */
  private boolean m_debug = false;

  public ColorPicker()
  {
    super();
    try
    {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
      SwingUtilities.updateComponentTreeUI(this);
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
    }
  }

  /**
   * method to display a color picker dialog.
   *
   * @param the title for the dialog
   * @param the initial color to show in the picker
   * @return the selected color (or null)
   */
  public Color showColorPicker(String dialogTitle, Color initialColor)
  {
    log("showColorPicker(" + dialogTitle + "," + initialColor + ")");
    Color retColor = JColorChooser.showDialog(this,dialogTitle,initialColor);
    log("showColorPicker() Selected => " + retColor);
    return retColor;
  }

  /**
   * Sets debug mode
   */
  public void setDebug(boolean debugMode)
  {
    log("setDebug(" + debugMode + ")");
    m_debug = debugMode;
  }

  /**
   * Gets debug mode
   */
  public boolean getDebug()
  {
    log("getDebug()");
    return m_debug;
  }

  /**
   * Convenience method to display an message out to the console.  Examines the
   * m_Debug  setting to see if it should output the message.  Outputs message in
   * format classname : msg
   *
   * @param the message to be output
   */
  public void log(String msg)
  {
    if(m_debug)
        System.out.println(CLASSNAME + ": " + msg);
  }
}