package oracle.forms.demos.uploadserver;

import sun.misc.BASE64Decoder;
import java.io.FileOutputStream;
import java.io.DataOutputStream;
import java.io.DataInputStream;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.zip.*;
import java.io.File;
import java.io.FileWriter;
import java.lang.Exception;

/**
 * This class performs the task of assisting with the conversion of BASE64 encoded
 * data and the subsequent storage in a file.  This class was designed primarily
 * for use from the Forms 6<i>i</i> Java Importer in conjunction with the
 * FileUploader JavaBean.
 * <P>The FileUploader JavaBean returns BASE64 encoded data in
 * a series of 4K chunks which need to be reassembled into a contiguous string for
 * use in the decoding process.  This class enables a Form to store the chunks
 * of data it receives with the addChunk() method and then to decode the chunks
 * using the decodeBuffer() method.
 * <P>The Byte stream of the uploaded file may have been compressed before uploading
 * so the decodeBuffer() method takes care of that as well
 * <P><u>Change History</u></P>
 * drmills 23-Oct-2001 - Code added to allow unicode filenames. Better error handling
 *
 * @version 9.0.1
 * @date 23-Oct-2001
 * @author Steve Button
 * @author Duncan Mills
 * @author JongHyuk Song
 */

public class FormsDecoder
{

  public static String FormsDecoderVersion = "9.0.3";
  /**
   * Used to store the chunks as they are added.
   */
  private StringBuffer buffer;

  private String mLogFileName = null;

  /**
   * The BASE64 encoder used to convert the data back to it's original format.
   */
  private BASE64Decoder DECODER = new BASE64Decoder();

  /**
   * Create a new FormsDecoder object.  This operation initialises the
   * buffer used to store the fragments.
   */
  public FormsDecoder()
  {
    buffer = new StringBuffer();
  }

  /**
   * Appends a chunk of data to the end of the buffer.
   *
   * @param data the chunk of data to be appended to the buffer.
   */
  public void addChunk(String data) throws FormsDecoderException
  {
    log("Adding chunk\n");
    buffer.append(data);
  }


  /**
   * Convert the data that has been added to the buffer via the addChunk method
   * to it's original format using sun.misc.BASE64Decoder and write it out to the
   * specified filename.
   *
   * @param filename the file to create for the decoded data.
   * @param filesize the size of the uploaded ilef
   * @param isCompressed indicator that we need to unzip the file
   * @throws IOException
   * @throws FormsDecoderException
   */
  public void decodeBuffer(String directoryName, String fileName, int filesize, boolean isCompressed) throws IOException, FormsDecoderException
  {
    try
    {
      log("Decoding..\n");
      fileName = java.net.URLDecoder.decode(fileName);
      log("decodes Filename..\n");
      if(buffer.length()==0)
        throw(new FormsDecoderException("no data in the buffer to decode"));

      FileOutputStream fos = new FileOutputStream(new File(directoryName,fileName));
      log("Opened outputstream ..\n");
      byte[] decoded = DECODER.decodeBuffer(buffer.toString());
      log("Decoded buffer ..\n");
      byte[] decompressedArray = new byte[0];
      if (isCompressed)
      {
        log("About to decompress ..\n");
        ByteArrayInputStream bais = new ByteArrayInputStream(decoded);
        GZIPInputStream gis = new GZIPInputStream(bais);
        DataInputStream dis = new DataInputStream(gis);
        decompressedArray = new byte[filesize];
        dis.readFully(decompressedArray);
        fos.write(decompressedArray);
        log("decompress done ..\n");
      }
      else
      {
        fos.write(decoded);
      }
      fos.close();
      log("fos closed ..\n");

      // reset the variable to try and free them up
      decoded=new byte[0];
      decompressedArray = new byte[0];
      buffer=new StringBuffer();
      Runtime.getRuntime().runFinalization();
      log("Finished ..\n");
    }
    catch (Exception ex)
    {
      log("DecodeBuffer exception\n" + ex.toString() + "\nDirectory: " + directoryName + "\nFile: " + fileName + "\nSize: " + filesize + "\n");
      throw new FormsDecoderException("decodeBuffer exception: " + ex.toString());
    }
  }

  /**
   * Define a debugging log file - setting this with the name of an o/s file
   * implicitly switches debugging on
   */
  public void setLogFileName(String logFile)
  {
    mLogFileName = logFile;
  }

  /**
   * Log a debugging message if a log file name has been registered
   */
  private void log(String logMessage)
  {
    if (mLogFileName != null)
    {
      try
      {
        FileWriter writer = new FileWriter(mLogFileName,true);
        writer.write(logMessage + "\n");
        writer.close();
      }
      catch (IOException ioe)
      {
        System.out.println("Logging> IO Exception");
      }
    }
  }


  /**
   * Test - Throws a FormsDecoder exception
   */
  public void throwEx() throws FormsDecoderException
  {
    throw new FormsDecoderException("test exception");
  }


}
