package oracle.forms.demos.enhancedItems;

import oracle.ewt.lwAWT.lwText.LWTextArea;
import java.awt.event.*;

/**
 * <H3><FONT COLOR="#336699">InsertReplace Pluggable Java Component Helper Class for Oracle Forms</FONT></H3>
 * This class is used by the InsertReplaceTextArea Pluggable Java Component for Oracle Forms, that implements
 * a method for toggling between insert and replace mode much like a text editor or a character mode application
 *
 * This class actually provides a private implemenation of the ewt LWTextArea which can do ReplaceMode
 * <br>
 * For more PJC Demos and White Papers see <a href="http://otn.oracle.com/products/forms" target="_blank">Forms on OTN</a>
 *
 * @version 9.0 06/Nov/2001<br>
 *
 * @author Duncan Mills
 */
public class InsertReplaceLWTextArea extends LWTextArea
{

  /**
   * Constructor for the InsertReplaceLWTextArea
   */

  public InsertReplaceLWTextArea()
  {
    super();
  }

  /**
   * A function that is called by the underlying control to handle keystrokes
   * Here, if replace mode is specified then it will replace the value with the
   * current keystroke
   * @param event the KeyEvent that is passed to us to process
   */

   protected void processKeyEvent(KeyEvent event)
   {
     if (event.getID() == KeyEvent.KEY_TYPED && InsertReplaceHelper.getReplaceMode())
     {
      int caretPos = this.getCaretPosition();
      if (caretPos != this.getSelectionStart())
      {
        //in this case don't replace manually as the user has made a selection
        //and the default functionality will replace for us
        super.processKeyEvent(event);
      }
      else
      {
        if (caretPos < this.getTextLength())
          //this is where the work actually happens
          //last chance saloon to exclude keysrokes
          // Backspace for instance seems to get through
          switch (event.getKeyChar())
          {
            case 8:
            case 9:
              super.processKeyEvent(event);
              break;
            default:
              this.replaceRange("" + event.getKeyChar(),caretPos,caretPos+1);
          }
        else
         super.processKeyEvent(event);
      }
     }
     else if (event.getID() == KeyEvent.KEY_PRESSED && event.getKeyCode() == KeyEvent.VK_INSERT && event.getModifiers() == 0)
     {
       if (InsertReplaceHelper.getReplaceMode())
       {
         InsertReplaceHelper.setReplaceMode(false);
       }
       else
       {
         InsertReplaceHelper.setReplaceMode(true);
       }
       super.processKeyEvent(event);
     }
     else
     {
       super.processKeyEvent(event);
     }
   }
}