package oracle.forms.jdapi.util;

import java.util.*;
import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * Servlet to recursively browse Jdapi metadata in a web page
 *
 * @author Mark Warner (mark.warner@oracle.com)
 * @author Dan Pusceddu (dan.pusceddu@oracle.com)
 */
public class FormsBrowser extends HttpServlet
{
  /**
   * Reference the printwriter from the Http response object
   */
  protected PrintWriter m_out;

  /**
   * The forms object currently being rendered (may be null)
   */
  private String m_currentObject;

  /**
   * Contains the current formatted list of Forms Objects
   */
  private String m_objectListHtml;

  /**
   * Contains the current formatted list of current object's parents
   */
  private String m_parentListHtml;

  /**
   * Contains the current formatted list of current object's children
   */
  private String m_childListHtml;

  /**
   * Contains the current formatted list of current object's metaproperties
   */
  private String m_propertyListHtml;

  private MetaBrowser m_MetaBrowser = new MetaBrowser();

  /**
   * Overridden implementation of HttpServlet doGet() method
   */
  public void doGet(HttpServletRequest req,
                    HttpServletResponse res)
    throws ServletException, IOException
  {
    // get the response's printwriter
    m_out = res.getWriter();

    // update the servlet state
    m_currentObject = req.getParameter("object");

    Enumeration e = m_MetaBrowser.getGenericList();
    m_objectListHtml = renderVerticalList(e, "Forms Objects");

    if (m_currentObject != null)
    {
System.out.println("5a");
      e = m_MetaBrowser.getParents(m_currentObject);
System.out.println("5b");
      m_parentListHtml = renderHorizontalList(e, "Object Parents");

System.out.println("5c");
      e = m_MetaBrowser.getChildren(m_currentObject);
System.out.println("5d");
      m_childListHtml = renderHorizontalList(e, "Object Children");

System.out.println("5e "+m_currentObject.toString());
      e = m_MetaBrowser.getMetaProperties(m_currentObject);
System.out.println("5f");
      m_propertyListHtml = renderPropertyList(e, "Object Properties");
    }

    res.setContentType("text/html");
    renderHeader();

    if (m_currentObject != null)
    {
      renderBody();
    }
    else
    {
      renderFrontPage();
    }
    renderFooter();
  }


  /**
   * Render the dynamic part of the page body
   */
  protected void renderBody()
  {
    m_out.println("<body>");
    m_out.println("<table class=\"layout\" width=100%>");
    m_out.println("<tr>");
    // Title Cell
    m_out.println("<td>");
    m_out.println("<a href=\"FormsBrowser\"><img alt=\"Demos\" align=\"middle\" src=\"formsdemo/images/banner9iDS.gif\"></a>");
    m_out.println("</td>");
    m_out.println("<td class=\"titlespan\">");
    m_out.println("Oracle Forms Developer Metadata Browser");
    m_out.println("</td>");
    m_out.println("</tr>");
    m_out.println("<tr>");
    // cell containing the list of objects
    m_out.println("<td width=180 rowspan=4>");

    m_out.println("<p>");

    m_out.println(m_objectListHtml);

    m_out.println("</p>");

    // close cell containing objects
    m_out.println("</td>");

    // cell containing current object title
    m_out.println("<td><span class=\"currobj\">" +
                                  m_currentObject + "</span><hr>");
    m_out.println("</td>");
    m_out.println("</tr>");

    m_out.println("<tr>");
    // cell containing current object parents
    m_out.println("<td><b>Object Parents:</b><br>");
    m_out.println(m_parentListHtml);
    m_out.println("<hr></td>");
    m_out.println("</tr>");

    m_out.println("<tr>");
    // cell containing current object children
    m_out.println("<td><b>Object Children:</b><br>");
    m_out.println(m_childListHtml);
    m_out.println("<hr></td>");
    m_out.println("</tr>");

    m_out.println("<tr>");
    // cell containing current object children
    m_out.println("<td><b>Object Property Metadata:</b><br>");
    m_out.println(m_propertyListHtml);
    m_out.println("</td>");
    m_out.println("</tr>");
    m_out.println("</table>");
    m_out.println("</body>");

  }

  /**
   * Render the page Header
   */
  protected void renderHeader()
  {
    m_out.println("<html>");
    m_out.println("<head>");

    // css rules used by the page
    m_out.println("<link rel=\"stylesheet\" type=\"text/css\" href=\"formsdemo/doc/metabrowser.css\">");
    m_out.println("<title>Forms Metadata Browser</title>");
    m_out.println("<script language=\"JavaScript\" src=\"formsdemo/doc/metabrowser.js\"></script>");
    m_out.println("</head>");
  }

  /**
   * Render the page footer
   */
  protected void renderFooter()
  {
    m_out.println("</html>");
  }

  /**
   * Render a default page if there's no current object
   */
  protected void renderFrontPage()
  {
    m_out.println("<body>");
    m_out.println("<table class=\"layout\" width=100%>");
    m_out.println("<tr>");
    m_out.println("<tr>");
    // Title Cell
    m_out.println("<td>");
    m_out.println("<a href=\"FormsBrowser\"><img alt=\"Demos\" align=\"middle\" src=\"formsdemo/images/banner9iDS.gif\"></a>");
    m_out.println("</td>");
    m_out.println("<td class=\"titlespan\">");
    m_out.println("Oracle<i>i</i> Forms Developer Metadata Browser");
    m_out.println("</td>");
    m_out.println("</tr>");

    // cell containing the list of objects
    m_out.println("<td width=180>");
    m_out.println("<p>");
    m_out.println(m_objectListHtml);
    m_out.println("</p>");

    // close cell containing objects
    m_out.println("</td>");

    // Default page (should be template-driven
    m_out.println("<td>");
    m_out.println("<b>Oracle Forms Developer Metadata Browser Servlet</b><hr>");

    m_out.println("<p>The list of links on the left is the entire set of objects in the forms model. Selecting an object will show that object's possible parent objects, child objects and a complete set of meta properties. You can then continue drilling down on the model hierarchy.</p>");
    m_out.println("<p>This application was written by the UK Forms team. It is implemented as a java servlet and is an example of using the forms Java Design-time API to access the metadata of forms objects on the fly. ");
    m_out.println("<p>Using jdapi it is possible to access all the metadata which comprises the forms model and examine the possible properties of any object without having to run the Forms Builder tool, and without having to instantiate actual forms objects.</p>");
    m_out.println("<hr>");

    m_out.println("</td>");
    m_out.println("</tr>");
    m_out.println("</table>");
    m_out.println("</body>");
  }

  /**
   * Render a list of links in vertical format
   */
  protected String renderVerticalList(Enumeration listMembers, String title)
  {
    StringBuffer list = new StringBuffer();

    list.append("<b>" + title + "</b><br>\n");

    if (!listMembers.hasMoreElements())
    {
      list.append("This object has no information for " + title);
    }
    while (listMembers.hasMoreElements())
    {
      String element = (String)listMembers.nextElement();
      if (element.equals(m_currentObject))
      {
        list.append("<li><font class=\"currspan\">" + element + "</font></li>\n");
      }
      else
      {
        list.append("<li><a href=\"/forms/servlet/FormsBrowser?object=" + element +
                    "\">" + element + "</a></li>\n");
      }
    }

    return list.toString();
  }

  /**
   * Render a list of links in horizontal format
   */
  protected String renderHorizontalList(Enumeration listMembers, String title)
  {
    StringBuffer list = new StringBuffer();

    //list.append("<b>" + title + "</b>\n");

    if (!listMembers.hasMoreElements())
    {
      list.append("This object has no " + title);
    }
    while (listMembers.hasMoreElements())
    {
      String element = (String)listMembers.nextElement();
      if (element.equals(m_currentObject))
      {
        list.append("[<span class=\"currspan\">" + element + "</span>]&nbsp;\n");
      }
      else
      {
        list.append("[<a href=\"FormsBrowser?object=" + element +
                  "\">" + element + "</a>]&nbsp;\n");
      }
    }

    return list.toString();
  }

  protected String renderPropertyList(Enumeration listMembers, String title)
  {
    StringBuffer list = new StringBuffer();

    if (!listMembers.hasMoreElements())
    {
      list.append("This object has no information for " + title);
    }
    else
    {
      list.append("<table width=100% border=1 cellpadding=1 cellspacing=0 align=\"left\" class=\"proptable\">\n" +
                  "<tr class=\"tableheader\">\n" +
                  "<td>Property Name</td>\n" + "<td align=\"center\">Property Type</td>\n" +
                  "<td align=\"center\">Can Set</td>\n" + "<td align=\"right\">Property ID</td>\n" +
                  "<td align=\"left\">Description</td>\n" + "</tr>\n");
      while (listMembers.hasMoreElements())
      {
        PropertyContainer pc = (PropertyContainer)listMembers.nextElement();
        list.append("<tr>\n");
        if(pc.getPropertyType().equals("Integer") && null != pc.getAllowedValues() )
        {
          String divName = "d" + String.valueOf(pc.getPropertyNum() );
          list.append("<td><a href=\"javascript:showHide('" + divName + "');\" "
                      + "onMouseOver=\"window.status='Show Allowed Values';return true;\" "
                      + "onMouseOut=\"window.status='';return true;\">"
                      + pc.getPropertyName() + "</a>"
                      + "<div id=\"" + divName + "\" class=\"propvals\" style=\"display:none\">"
                      + renderValues(pc.getAllowedValues() )
                      + "</div></td>\n");
        }
        else
        {
          list.append("<td>" + pc.getPropertyName() + "</td>\n");
        }
        list.append("<td align=\"center\">" + pc.getPropertyType() + "</td>\n");
        list.append("<td align=\"center\">" + renderBoolean(pc.allowSet()) + "</td>\n");
        list.append("<td align=\"right\">" + pc.getPropertyNum() + "</td>\n");
        list.append("<td align=\"left\">" + pc.getPropertyDesc() + "</td>\n");
        list.append("</tr>\n");
      }
      list.append("</table>\n");
    }
    return list.toString();
  }

  private String renderValues(HashMap vals)
  {
    StringBuffer ret = new StringBuffer();
    for(int n = 0; n < vals.size(); n++)
    {
      ret.append(n + " " + vals.get(new Integer(n)) + "<br>");
    }
    return ret.toString();
  }

  private String renderBoolean(boolean b)
  {
    if(b)
    {
      return "<div class=\"yes\">Yes</div>";
    }
    else
    {
      return "<div class=\"no\">No</div>";
    }
  }
}
