package oracle.forms.jdapi.util;

import java.util.HashMap;

/**
 * Container class for a number of different properties.
 *
 * @author Mark Warner (mark.warner@oracle.com)
 * @author Dan Pusceddu (dan.pusceddu@oracle.com)
 */
public class PropertyContainer
{
  private String m_propertyName;
  private String m_propertyDesc;
  private String m_propertyType;
  private int m_propertyNum;
  private boolean m_allowSet;
  private boolean m_allowGet;
  private HashMap m_allowedValues;

  /**
   * Constructor for the container.
   */
  public PropertyContainer()
  {
  }

  /**
   * Constructor for the container. Sets the containers member to be those given.
   *
   * @param String pName - property name to be held in the container.
   * @param String pType - property type to be held in the container.
   * @param int pnum - property numberto be held in the container.
   */
  public PropertyContainer(String pName, String pType, String pDesc, int pnum, HashMap vals, boolean g, boolean s)
  {
    setPropertyName(pName);
    setPropertyType(pType);
    setPropertyNum(pnum);
    setPropertyDesc(pDesc);
    setAllowedValues(vals);
    allowGet(g);
    allowSet(s);
  }

  /**
   * Returns the property name held in the container as a string.
   *
   * @return String
   */
  public String getPropertyName()
  {
    return m_propertyName;
  }

  /**
   * Get the property description held in the container
   *
   * @param String x
   */
  public String getPropertyDesc()
  {
    return m_propertyDesc;
  }

  /**
   * Returns the property type held in the container as a string.
   *
   * @return String
   */
  public String getPropertyType()
  {
    return m_propertyType;
  }

  /**
   * Returns the property number held in the container as an int.
   *
   * @return int
   */
  public int getPropertyNum()
  {
    return m_propertyNum;
  }

  /**
   * Is the Property "get"able?
   *
   * @return boolean
   */
  public boolean allowGet()
  {
    return m_allowGet;
  }

  /**
   * Is the Property "set"able?
   *
   * @return boolean
   */
  public boolean allowSet()
  {
    return m_allowSet;
  }

  /**
   * Get allowed properties
   *
   * @return int[]
   */
  public HashMap getAllowedValues()
  {
    return m_allowedValues;
  }

  /**
   * Set the property name held in the container
   *
   * @param String x
   */
  public void setPropertyName(String x)
  {
    m_propertyName = x;
  }

  /**
   * Set the property description held in the container
   *
   * @param String x
   */
  public void setPropertyDesc(String x)
  {
    m_propertyDesc = x;
  }

  /**
   * Set the property type held in the container
   *
   * @param String x
   */
  public void setPropertyType(String x)
  {
    m_propertyType = x;
  }

  /**
   * Set the property number held in the container
   *
   * @param int x
   */
  public void setPropertyNum(int x)
  {
    m_propertyNum = x;
  }

  /**
   * Set the Property as "get"able.
   *
   * @param boolean b
   */
  public void allowGet(boolean b)
  {
    m_allowGet = b;
  }

  /**
   * Set the Property as "set"able.
   *
   * @param boolean b
   */
  public void allowSet(boolean b)
  {
    m_allowSet = b;
  }

  /**
   * Set allowed properties
   *
   * @param int[] a
   */
  public void setAllowedValues(HashMap a)
  {
    m_allowedValues = a;
  }

}
