DROP TABLE hl_product_images;
DROP TABLE hl_products;
DROP TABLE hl_suppliers;

DROP SEQUENCE hl_suppliers_s;
DROP SEQUENCE hl_products_s;

CREATE TABLE hl_suppliers
(
  supplier_id            NUMBER         NOT NULL
, supplier_name          VARCHAR2(20)   NOT NULL
, supplier_password      VARCHAR2(20)   NOT NULL
, poc_name               VARCHAR2(40)       NULL
, poc_phone_number       VARCHAR2(40)       NULL
, poc_email_address      VARCHAR2(40)       NULL
, poc_address_line1      VARCHAR2(40)       NULL
, poc_address_line2      VARCHAR2(40)       NULL
, poc_city               VARCHAR2(40)       NULL
, poc_state              VARCHAR2(40)       NULL
, poc_postal_code        VARCHAR2(40)       NULL
, poc_country            VARCHAR2(40)       NULL
, bank_name              VARCHAR2(40)       NULL
, bank_routing_number    NUMBER             NULL
, account_number         NUMBER             NULL
)
/

ALTER TABLE hl_suppliers ADD CONSTRAINT hl_supp_pk PRIMARY KEY (supplier_id)
/

ALTER TABLE hl_suppliers ADD CONSTRAINT hl_supp_uk1 UNIQUE (supplier_name)
/

CREATE TABLE hl_products
(
  product_id             NUMBER         NOT NULL
, supplier_id            NUMBER         NOT NULL
, product_name           VARCHAR2(30)   NOT NULL
, description            VARCHAR2(255)      NULL
, category               VARCHAR2(30)   NOT NULL
, manufacturer           VARCHAR2(50)       NULL
, handling_instructions  VARCHAR2(300)      NULL
, price                  NUMBER             NULL
, currency               VARCHAR2(3)        NULL
, weight                 NUMBER             NULL 
, weight_uom             VARCHAR2(10)       NULL
, volume                 NUMBER             NULL
, volume_uom             VARCHAR2(10)       NULL
)
/

ALTER TABLE hl_products ADD CONSTRAINT hl_prod_pk PRIMARY KEY (product_id)
/

ALTER TABLE hl_products ADD CONSTRAINT hl_prod_supp_fk
FOREIGN KEY (supplier_id) REFERENCES hl_suppliers (supplier_id)
/

CREATE TABLE hl_product_images
(
  product_id             NUMBER         NOT NULL
, content                LONG RAW       NOT NULL
)
/

ALTER TABLE hl_product_images ADD CONSTRAINT hl_img_prod_fk
FOREIGN KEY (product_id) REFERENCES hl_products (product_id)
/

CREATE SEQUENCE hl_products_s  MINVALUE 1000 INCREMENT BY 1;

CREATE SEQUENCE hl_suppliers_s MINVALUE 1000 INCREMENT BY 1;

