rem
rem Header: oe8_main.sql 09-jan-01
rem
rem Copyright (c) 2001, Oracle Corporation.  All rights reserved.  
rem
rem Owner  : ahunold
rem
rem NAME
rem   oe8_main.sql - Main script for oe8 schema, including OC subschema 
rem
rem DESCRIPTON
rem   Creates and populated the Order Entry (oe8) and Online
rem   Catalog (OC) Sample Schema
rem
rem NOTES
rem   Run as SYS or SYSTEM
rem   Prerequisites:
rem     Tablespaces present
rem     Database enabled for Spatial and XML
rem
rem MODIFIED   (MM/DD/YY)
rem   ahunold   10/03/01 - 8i version
rem   drmills   11/20/01 - Forms 9i Demos - add right to create procedure

SET ECHO OFF

PROMPT 
PROMPT specify password for oe8 as parameter 1:
DEFINE pass     = &1
PROMPT 
PROMPT specify default tablespeace for oe8 as parameter 2:
DEFINE tbs      = &2
PROMPT 
PROMPT specify temporary tablespace for oe8 as parameter 3:
DEFINE ttbs     = &3
PROMPT 
PROMPT specify password for HR as parameter 4:
DEFINE passhr   = &4
PROMPT 
PROMPT specify password for SYS as parameter 5:
DEFINE pass_sys = &5
PROMPT 
PROMPT specify path for log files as parameter 6:
DEFINE log_path = &6
PROMPT
PROMPT specify database as parameter 7:
DEFINE database = &7
PROMPT

-- The first dot in the spool command below is 
-- the SQL*Plus concatenation character

DEFINE spool_file = &log_path.oe8_main.log
SPOOL &spool_file

-- Dropping the user with all its objects

DROP USER oe8 CASCADE;

REM =======================================================
REM create user
REM 
REM The user is assigned tablespaces and quota in separate
REM ALTER USER statements so that the CREATE USER statement
REM will succeed even if the demo and temp tablespaces do
REM not exist.
REM =======================================================

CREATE USER oe8 IDENTIFIED BY &pass;

ALTER USER oe8 DEFAULT TABLESPACE &tbs QUOTA UNLIMITED ON &tbs;

ALTER USER oe8 TEMPORARY TABLESPACE &ttbs;

GRANT CONNECT TO oe8;
GRANT RESOURCE TO oe8;
GRANT CREATE PROCEDURE TO oe8;
GRANT CREATE MATERIALIZED VIEW  TO oe8;
GRANT QUERY REWRITE             TO oe8;

REM =======================================================
REM grants from sys schema
REM =======================================================

CONNECT sys/&pass_sys@&database AS SYSDBA;  
GRANT execute ON sys.dbms_stats TO oe8;


REM =======================================================
REM grants from hr8 schema
REM =======================================================

CONNECT hr8/&passhr@&database;
GRANT REFERENCES, SELECT ON employees TO oe8;
GRANT REFERENCES, SELECT ON countries TO oe8;
GRANT REFERENCES, SELECT ON locations TO oe8;
GRANT SELECT ON jobs TO oe8;
GRANT SELECT ON job_history TO oe8;
GRANT SELECT ON departments TO oe8;


REM =======================================================
REM create oe8 schema (order entry)
REM =======================================================

CONNECT oe8/&pass@&database
ALTER SESSION SET NLS_LANGUAGE=American;
ALTER SESSION SET NLS_TERRITORY=America;
@@oe8_cre
@@oe8_p_pi  
@@oe8_p_pd
@@oe8_p_whs                      
@@oe8_p_cus
@@oe8_p_ord
@@oe8_p_itm 
@@oe8_p_inv                               
@@oe8_views
@@oe8_comnt
@@oe8_idx
@@oe8_analz

spool off
