/* Copyright (c) Oracle Corporation 2000.  All Rights Reserved. */

package oracle.forms.demos.beans;


import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.*;

import oracle.ewt.lwAWT.*;

/**
 * <H3><FONT COLOR="003399">Oracle Forms Server 9i JavaBean Example</FONT></H3>
 *
 * This JavaBean provides displays a digital clock which shows the current time
 * on the client computer.  The digital clock can be set to display in all
 * the standard Java colors and fonts.
 *
 * In addition to displaying the time, this JavaBean also is able to responds
 * to the user dragging the mouse (moving the mouse over the component with a
 * mouse button depressed).  When the user performs this action, this component
 * will move the time display to follow the mouse cursor.
 *
 * @version 1.0
 * @author sbutton 03/17/00
 */

public class TickerApplet
       extends LWComponent
       implements Runnable
{

  /*
   * The X offset for the string to be painted at, default of 5;
   */
  private int x = 5;
  /*
   * The Y offset for the string to be painted at, default of 15;
   */
  private int y = 15;
  /*
   * The width of the string to be displayed
   */
  private int swidth = 0;
  /*
   * The height of the string to be displayed
   */
  private int sheight = 0;
  /*
   * The height of the component, defaults to 25
   */
  private int height=25;
  /*
   * The width of the component, defaults to 100
   */
  private int width=100;
  /*
   * The fontsize to use for the component, defaults to 16
   */
  private int fontSize = 16;
  /*
   * The font to use for the component, defaults to Helvetica BOLD
   */
  private Font font = new Font("Helvetica", Font.BOLD, fontSize);
  /*
   * The background to use for the component, defaults to BLACK
   */
  private Color background = Color.black;
  /*
   * The foreground to use for the component, defaults to GREEN
   */
  private Color foreground = Color.green;
  /*
   * The offscreen-image to use for double buffering purposes
   */
  private Image memImage = null;
  /*
   * The offscreen-graphics object to use for double buffering purposes
   */
  private Graphics offScr = null;
  /*
   * The message to display
   */
  private static String fullMessage = "TIME IS ... ";
  /*
   * The date formater to turn an ugly date into a nice one
   */
  private DateFormat df = DateFormat.getTimeInstance();
  /**
   * The thread used by this component
   */
  private Thread tickerThread = null;

  private boolean m_continue = true;

  /**
   * The constructor for this JavaBean.  Initialises the mousedrag opereration
   */
  public TickerApplet()
  {
    initMouseDrag();
  }


  /**
   * Sets the Font size to use
   *
   * @param the size of the font to display
   */
  public void setFontSize(int fs)
  {
    this.fontSize = fs;
    font = new Font("Helvetica", Font.BOLD, this.fontSize);
  }

  public int getFontSize()
  {
    return this.fontSize;
  }

  /**
   * Sets the Font to use
   *
   * @param the font to display
   */
  public void setFont(Font f)
  {
    this.font = f;
  }

  /**
   * Gets the Font currently being used
   *
   * @return the font being displayed
   */
  public Font getFont()
  {
    return font;
  }

  /**
   * Sets the background color to use
   *
   * @param the background color to use
   */
  public void setBackground(Color background)
  {
    this.background = background;
  }

  /**
   * Gets the background color currently being used
   *
   * @return the background color being displayed
   */
  public Color getBackground()
  {
    return background;
  }

  /**
   * Sets the foreground color to use
   *
   * @param the foreground color to use
   */
  public void setForeground(Color foreground)
  {
    this.foreground = foreground;
  }

  /**
   * Gets the foreground color currently being used
   *
   * @return the background color being displayed
   */
  public Color getForeground()
  {
    return foreground;
  }

  /**
   * Sets the X offset for the display string
   *
   * @param the X offset
   */
  public void setX(int x)
  {
    this.x = x;
  }

  /**
   * Gets the X offset being used
   *
   * @return the X offset
   */
  public int getX()
  {
    return this.x;
  }

  /**
   * Sets the Y offset for the display string
   *
   * @param the Y offset
   */
  public void setY(int y)
  {
    this.y = y;
  }

  /**
   * Gets the Y offset being used
   *
   * @return the Y offset
   */
  public int getY()
  {
    return this.y;
  }

  /**
   * Set the size of the component, overriden from the super class so that the
   * calcOffsets method can be called to work out the size of the component and
   * where the string needs to be placed.
   *
   * @param the desired size of the component
   */
  public void setSize(Dimension dim)
  {
    System.out.println(getClass().getName() + ":setSize(" + dim.toString() + ")");
    super.setSize(dim);
    this.width=dim.width;
    this.height=dim.height;
    calcOffsets();
    repaint();
  }

  /*
  public void init()
  {
    System.out.println(getClass().getName() + ": init()");
  }
  */

  /**
   * Obtains the current time in a nice format using the DateFormat class.
   * Will return a timestring of HH:MM:SS
   *
   * @return the current time in HH:MM:SS format
   */
  public String getTimeString()
  {
    df = DateFormat.getTimeInstance();
    Date d = new Date();
    return(df.format(d));
  }

  /**
   * Calculates the offsets to use when the string is displayed.  This is used
   * to center the string when it is first displayed and is only called
   * once.  If the user performs a mouse-drag then the values set by the
   * method will be overriden and the string will no longer be centered.
   *
   */
  public void calcOffsets()
  {
    String time = getTimeString();

    // calculate the string attributes, height and width
    FontMetrics fm = this.getFontMetrics(this.font);
    swidth = fm.stringWidth(getTimeString());
    sheight = fm.getHeight();

    // set the X and Y positions
    x = (this.width/2) - (swidth/2);
    y = (this.height/2) + (sheight/2);

    System.out.println(getClass().getName() + "calcOffsets() " + time + " x=" + x + ", y=" + y);
  }


  /**
   * Start the components actions by initialising the thread and setting it free
   */
  public void start()
  {
    System.out.println(getClass().getName() + ": start()");
    tickerThread = new Thread(this);
    m_continue = true;
    tickerThread.start();
  }

  /**
   * Stop the component's actions by killing the thread and setting it back
   * to null for future use.
   */
  public void stop()
  {
    System.out.println(getClass().getName() + ": stop()");
    tickerThread = null;
    m_continue = false;
  }

  /**
   * Paint the component using the current attributes of the
   * colors, fonts and string to be displayed
   *
   * @param the graphics context to draw on
   */
  public void paint(Graphics g)
  {
    g.setColor(background);
    g.fillRect(0, 0, width, height);
    g.setColor(foreground);
    g.setFont(font);
    g.drawString(fullMessage, x, y);
  }

  /**
   * The guts of the component.  This is where the actions are performed
   * to actually have the component perform actions on the screen.  Creates the
   * double buffer graphics context and images, obtains the time and then gets
   * it painted.  Goes to sleep for 1000ms which should see it only repaint the
   * display every second.
   */
  public void run()
  {
    try
    {
      System.out.println(getClass().getName() + ": run()");
      while (m_continue)
      {
        // obtain the time
        fullMessage = getTimeString();

        // setup the double buffering
        memImage = this.createImage(width,height);
        Graphics offScreenGraphic = memImage.getGraphics();

        // paints the double buffered graphics context
        paint(offScreenGraphic);

        // display our double buffered friend
        Graphics onScreen = this.getGraphics();
        onScreen.drawImage(memImage, 0, 0, this);

        // zzzzz ..... zzzzz .....
  Thread.sleep(1000);
      }
    }
    catch (InterruptedException e)
    {
      e.printStackTrace();
    }
    catch (NullPointerException npe)
    {
    }
  }

  /**
   * <P>This methods sets up the component to be able to respond to the user
   * mouse drag events.  This method is called once and a mousedrag listener
   * is registered with the component.  When the user drags the mouse over this
   * component, the listener will be notified.</P>
   * <P>The mousedrag listener is subclasses of the mouse motion adapter and
   * the mousedragged method is overridden.  The mousedragged method obtains
   * the current [X,Y] position of the mouse and modifies the [X,Y] location
   * being used to display the string to reflect this.
   */
  public void initMouseDrag()
  {
    // add an anonymous adapter which subclasses the mousemotion adapater
    this.addMouseMotionListener(new MouseMotionAdapter()
    {
      final int border = 5;

      /**
       * Handles the mouseDragged event when it occurs.
       *
       * @param the mouse event details which contain the [X,Y] positions which
       * are used to remap the string display location
       */
      public void mouseDragged(MouseEvent me)
      {
        int mx = me.getPoint().x;
        int my = me.getPoint().y;

        x = mx;
        y = my;
        repaint();
      }
    });
  }

}















































