package oracle.forms.demos.beans;

import java.applet.*;
import java.util.*;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import javax.servlet.http.Cookie;
import oracle.forms.ui.VBean;
import oracle.forms.properties.*;
import oracle.forms.handler.IHandler;
import oracle.forms.ui.*;
import netscape.javascript.*;

/**
  @author: fnimphiu 02/02
*/
/**
 *
 * <H3><FONT COLOR="003399">Oracle Forms JavaBean Example</FONT></H3>
 *
 * A CookieHandler is a JavaBean that can be used from Forms to read and write
 * cookies in the Browser.
 * The Bean can be used to set and get cookies and supports both temporary and
 * persistant cookies
 * The value of the cookie is always is passed as a string to and from Forms.
 * To create a cookie with a key/value pair you will create and pass a delimited
 * with the delimiter between Key and value ":" and the delimiter between pairs
 * "/"
 * e.g. Pass "username:scott/password:tiger" to store
 *      username=scott and password=tiger as a cookie value.
 *
 * The CookieHandler Bean supports the following Custom Properties:
 *
 * <TABLE BORDER="1">
 * <TR>
 * <TD><code>normal</code></TD>
 * <TD><i>The default state of the hyperlink when the user has/had no interaction[s]
 * with it.</TD></TR>
 * <TR>
 * <TD><code>active</code></TD>
 * <TD><i>The state of the hyperlink when the user has moved the mouse over it.</TD></TR>
 * <TR>
 * <TD><code>visited</code></TD>
 * <TD><i>The state of the hyperlink when the user clicked on the hyperlink component
 * during this particular application session.  Will expire when the user ends the
 * application.</TD></TR>
 * </TABLE>
 *
 * <P>
 * The hyperlink will not perform any default actions when the user clicks on it.  Any
 * component that is interested in the actions of the hyperlink must register itself
 * as an <code>ActionListener</code> with the hyperlink.  The hyperlink will notify the interested
 * parties when it has been clicked through an <code>ActionEvent</code>
 * </P>
 *
 * @version 9.0  22/Feb/2002
 * @author Frank Nimphius
 * @author Duncan Mills
 */

public class CookieHandler extends VBean
{

  /**
   * The property registered to return value of the named cookie supplied
   * in the GETCOOKIE_NAME property
   * Used with GET_CUSTOM_PROPERTY() from the Form
   */
  public static final ID GETCOOKIE            = ID.registerProperty("GETCOOKIE");

  /**
   * The property registered to set the name of the cookie that GETCOOKIE will
   * return the value of
   * Used with SET_CUSTOM_PROPERTY() from the Form
   */
  public static final ID GETCOOKIE_NAME       = ID.registerProperty("GETCOOKIE_NAME");

  /**
   * The property registered to set cookie in the browser
   * Used with SET_CUSTOM_PROPERTY() from the Form
   */
  public static final ID SETCOOKIE            = ID.registerProperty("SETCOOKIE");

  /**
   * Forms property used to indicate that we want the utility to switch
   * Messaging on to the Java Console so we can see what is going on.
   * Used with SET_CUSTOM_PROPERTY() from the Form
   */
  public static final ID DEBUGMESSAGES        = ID.registerProperty("DEBUGMESSAGES");

  /**
   * The property registered to contain the error value if an error event is
   * raises
   */
  public static final ID COOKIE_ERROR_MESSAGE = ID.registerProperty("COOKIE_ERROR_MESSAGE");

  /**
   * The property that represents an error to be raised via When-Custom-Item-Event
   * if there is a problem with setting ot getting the cookie
   */
  public static final ID COOKIE_ERROR         = ID.registerProperty("COOKIE_ERROR");


  /**
   * The classname used for debugging purposes (in the log method).
   * use getClass().getName(); if you want the
   * full package name
   */
  private final String CLASSNAME = this.getDefaultName();

  /**
   * Member variable to hold the name of the Cookie that GETCOOKIE will be using
   */
  String m_cookieName   = "";

  /*
   m_debug to switch on/off debugging
   The debug message contains the code line first, helping to track errors in the Bean Code
   */
  boolean m_debug = true;


  /* JSObject variables */
  JSObject m_window=null;
  JSObject m_document=null;

  Applet m_applet;

  /*Forms settings*/
  IHandler m_handler;

  /*Browser type */
  boolean isMSIE= false;

  public CookieHandler()
  {
  }


  // IView methods
  ////////////////////////////////////////////////////////////////////////////
  /**
   * Implementation of IView interface
   * init is called when the bean is first instanciated
   * we use this to st up information such as the browser type and hooks into
   * the DOM
   * @see oracle.forms.ui.IView
   */

  public void init (IHandler handler)
  {
    try
    {
      super.init(handler);
      log("init()");
      m_handler = handler;

      // get the forms applet handler
      m_applet= m_handler.getApplet();

      // get a handle to the applet window
      m_window = JSObject.getWindow(m_applet);

      /*
        We need to know the Browsertype as this decides how to set the GMT time
        string for the expiry value. IE expects the expiry value to e UTC while Netscape
        wants GMT. We are looking for MSIE and if this string isn't found assume Netscape
      */

      JSObject navigator = (JSObject) m_window.getMember("navigator");
      String thisBrowser = (String) navigator.getMember("appVersion");

      if (thisBrowser.indexOf("MSIE")>-1){
          isMSIE=true;
      }
      else
      {
          isMSIE=false;
      }

      // For the Javascript we need a handle to the document
      m_document = (JSObject) m_window.getMember("document");

    }
    catch (Exception e)
    {
      System.out.println("Error Initialising CookieHander Bean");
      System.out.println(e.toString());
      System.out.println("Stack Trace\n-----------------------");
      e.printStackTrace();
    }
  }

  /**
   * Implementation of IView interface which sets a requested property to a given value
   * In this case we also use this call to execute the methods for setting and getting
   * cookies.
   *
   * @param property property to be set.
   * @param value value of the property id.
   * @return      true if the property could be set, false otherwise.
   * @see oracle.forms.ui.IView
   */

  public boolean setProperty(ID property, Object value)
  {
    if (property == DEBUGMESSAGES)
    {
      if (value.toString().equalsIgnoreCase("true"))
      {
        m_debug = true;
      }
      else
      {
        m_debug = false;
      }

      log("Debugging " + m_debug);
      return true;
    }
    else if (property == SETCOOKIE)
    {
      log("setProperty(SETCOOKIE," + value.toString()+ ")");
      setCookie(value.toString());
      return true;
    }
    else if (property == GETCOOKIE_NAME)
    {
      log("setProperty(GETCOOKIE_NAME," + value.toString()+ ")");
      m_cookieName = value.toString();
      return true;
    }
    else
    {
      return super.setProperty(property, value);
    }
  }

  /**
   * Implementation of IView interface which gets the value of a requested property
   * In this case you can get the current value of DEBUGMESSAGES
   *
   * @param property    property to be gotten.
   * @return value of the property
   * @see oracle.forms.ui.IView
   */
  public Object getProperty(ID property)
  {
    if ( property == DEBUGMESSAGES )
    {
      log("getProperty(DEBUGMESSAGES)=" + m_debug);
      return ""+ m_debug;
    }
    else if ( property == GETCOOKIE )
    {
      log("getProperty(GETCOOKIE)");
      return getCookie(m_cookieName);
    }
    else
    {
      return super.getProperty(property);
    }
  }

  // Cookie Handling methods
  ////////////////////////////////////////////////////////////////////////////

  /**
   * Function to retreive the named cookie from the browser
   *
   * @param cookie_name - Name of the cookie to get
   * @return value of the cookie
   */
  private String getCookie(String cookie_name)
  {

    /* the cookie_name is determined by the following search criteria:
      1. The name must match a string in the cookie
      2. After the name a "=" must be present at the position index+name.length
    */
    String cookieValue=(String)m_document.getMember("cookie");
    int startPos=0;
    int curPos=0;

    log("All Browser cookies accessible from this path: "+cookieValue);

    if ((cookieValue!="")&(cookieValue!=null)){

      // loop through cookie to find all matches of cookie_name. This can also be
      // partial matches e.g Frank appears in Frankfurt although not the correct match
      // when searching for a cookie amed Frank

        for(startPos = cookieValue.indexOf(cookie_name);startPos >-1;startPos = cookieValue.indexOf(cookie_name,curPos)) {

          // The start position becomes the current position
            curPos=startPos+1;
          // This condition is only entered if the search criteria match
          // 1. startPos >-1
          // 2. first character after cookie_name is '='
          if ((startPos >-1)&& (cookieValue.charAt(startPos+cookie_name.length())=='=')) {
            int endPos = cookieValue.indexOf(";",startPos);
            if (endPos==-1){
            endPos=cookieValue.length();
            }
            cookieValue=cookieValue.substring(startPos+cookie_name.length()+1,endPos);

            log("Value of named cookie: " + cookie_name + "=" + cookieValue);


            return URLDecoder.decode(cookieValue);
          }
          else{

          }
        }
        if (startPos<0){
        // no cookie of this name found
        log("No cookie with this name found");
        dispatchErrors("Cookie access denied!");
        }
    }
    else{
      // no cookie found
      log("No cookies found");
      dispatchErrors("Cookie access denied!");
    }
    return "";
  }

  /**
   * method to process the cookie string passed from Forms and turn it into a correctly
   * formatted cookie string for the browser.
   *@param the in string contains the value, the domain, the path, the expiry and the
           delimiter values. The following caseinsensitive label identify the passed values
     <ol>
     <li>COOKIENAME     Name of the cookie</li>
     <li>COOKIEDOMAIN   The domain for which the cookie is set. The domain name must have
                    at least two periods. Eg.: .oracle.com</li>
     <li>COOKIEPATH     The path from which an application that reads the cookie is downloaded.
                    E.g. / for the root directory of the domain</li>
     <li>COOKIEVALUE    Any character string up to 4k</li>
     <li>COOKIEEXPIRY   A number value representing a number of days that the cookie stays valid.
                    No expiry value means that a temporary cookie gets created</li>
     </ol>

     Each key value pair is separated by a semicolon (;). A valid in string looks like
     cookiename=mycookie;cookiedomain=.oracle.com;cookiepath=/;cokievalue=Forms rocks!;
  */

  private void setCookie (String in)
  {

    int startPos;
    int endPos;
    int currPos;

    //variables to hold the info parsed out of the supplied string
    String cookieName   = "";
    String cookieDomain = "";
    String cookiePath   = "/";
    String cookieValue  = "";
    String cookieExpiry = "0";
    String cookieString = "";


    // 1. Parse all the bits out of the Supplied String

    // get the name of the cookie
    startPos=in.toLowerCase().indexOf("cookiename");
    if (startPos < 0)
    {
      dispatchErrors("Cannot write cookie without a name. No cookie was set to the Browser");
    }
    else
    {
      // create the new start position
      startPos=in.indexOf("=",startPos);
      endPos=in.indexOf(";",startPos);
      // setting the cookie name
      cookieName=in.substring(startPos+1,endPos);
      log("Parsing Cookie String : cookieName= " + cookieName);

      // get the cookie value from in string
      if ((currPos=in.toLowerCase().indexOf("cookievalue"))>=0)
      {
        startPos=currPos;
        startPos=in.indexOf("=",startPos);
        endPos=in.indexOf(";",startPos);
        // set cookie value
        cookieValue=in.substring(startPos+1,endPos);
        log("Parsing Cookie String : cookieValue= " + cookieValue);
      }

      // get domain
      if ((currPos=in.toLowerCase().indexOf("cookiedomain"))>=0)
      {
        startPos=currPos;
        startPos=in.indexOf("=",startPos);
        endPos=in.indexOf(";",startPos);
        cookieDomain=in.substring(startPos+1,endPos);
        log("Parsing Cookie String : cookieDomain= " + cookieDomain);
      }

      // get Path
      if ((currPos=in.toLowerCase().indexOf("cookiepath"))>=0)
      {
        startPos=currPos;
        startPos=in.indexOf("=",startPos);
        endPos=in.indexOf(";",startPos);
        // set Path
        cookiePath=in.substring(startPos+1,endPos);
        log("Parsing Cookie String : cookiePath= " + cookiePath);
      }

      //get cookie expiry
      if ((currPos=in.toLowerCase().indexOf("cookieexpiry"))>=0)
      {
        startPos=currPos;
        startPos=in.indexOf("=",startPos);
        endPos=in.indexOf(";",startPos);
        // set cookie expiry
        cookieExpiry=in.substring(startPos+1,endPos);
        log("Parsing Cookie String : cookieExpiry= " + cookieExpiry);
      }
    }
    log("Parsing Cookie String complete");

    // 2. Convert the Expiry date to UTC/GMT
    cookieExpiry=getExpiryString(cookieExpiry);

    // 3. Create the correct Format for the cookie string
    cookieString=bakeCookie(cookieName,cookieValue,cookieExpiry,cookiePath,cookieDomain);
    log("Generated Cookie String: " + cookieString);

    // 4. Finally set the cookie - this is done differently for IE and NS
    if (isMSIE)
    {
      m_document.setMember("cookie",cookieString);
      log("IE Cookie Set");
    }
    else
    {
      // For Netscape the eval method must be chosen as testing has shown that
      // document.setMember doesn't work well
      m_document.eval("document.cookie=\""+cookieString+"\"");
      log("NS Cookie Set");
    }
  }



  /**
   * Method to manfacture the cookie. This method is used by both cookie types
   * @return correctly formatted and encoded cookie string for the browser
   */
  private String bakeCookie(String cookieName, String cookieValue, String cookieExpiry, String cookiePath, String cookieDomain)
  {

    String cookieString = "";

    // encode the cookie value replacing e.g. blanks by %20
    cookieString=cookieName+"="+cookieValue+"; ";

    // add expiry date
    if ((cookieExpiry!="")&&(cookieExpiry!=" ")&&(cookieExpiry!=null))
    {
      cookieString+=cookieExpiry+"; ";
    }

    // add the cookie Path if value != "/"
    if ((cookiePath!="/")&&(cookiePath!=""))
    {
      cookieString+="path="+cookiePath+"; ";
    }

    // add the domain if provided. A domain shouldn't be handled if empty or starting with a
    // space

    if ((cookieDomain!="") && (cookieDomain.charAt(0)!=' '))
    {
      int firstPeriodPos=cookieDomain.indexOf(".");

      if (firstPeriodPos>=0)
      {
        // check for second period
        if (cookieDomain.indexOf(".",firstPeriodPos)==-1)
        {
          //invalid domain//
          dispatchErrors("Provided domain is not valid (missing second period)");
        }
        else
        {
          // valid domain with at least two periods provided
          cookieString+="domain="+cookieDomain+";";
        }
      }
      else
      {
        dispatchErrors("Provided domain is not valid (missing periods)");
      }
    }
    return cookieString;
  }

  private String getExpiryString(String expiryDayNumber)
  {
    String cookieExpry="";
    try{

      int numberOfDays = Integer.parseInt(expiryDayNumber);
      if ((numberOfDays>0)){

        // get todays time and date
        Date today = new Date();
        long longToday = today.getTime();
        //get the offset days as a long number
        long longExpryOffset = numberOfDays*60*60*24*1000;
        // get the expiry date as along number
        long expiryDateLong=longToday+longExpryOffset;
        // get the GMT string of the expiryDate
        Date expiryDate = new Date(expiryDateLong);
        // set cookieExpry to a text representation of UTC time
        // create format name_of_day, day name_of_mont year hour:minute:second
        DateFormat expireDateFormat = new SimpleDateFormat("EE, dd MMM yyyy hh:mm:ss");
        // get date format and add UTC
        if (isMSIE){
          cookieExpry=" expires='"+(String)expireDateFormat.format(expiryDate)+" UTC'";
        }
        else{
          cookieExpry=" expires='"+(String)expireDateFormat.format(expiryDate)+" GMT'";
        }
      }
      else {
        cookieExpry=null;
      }

      return cookieExpry;
      }

    catch (NumberFormatException nfe){
      dispatchErrors("Value entered for expiration days is not a number");
    }

    return cookieExpry;
  }


  // Logging and error handling
  ////////////////////////////////////////////////////////////////////////////

  /**
   * Method to log errors and return them to
   * Forms as a custom Event
   * @param message to be sent
   */
  private void dispatchErrors(String message)
  {
    log(message);
    try{
      CustomEvent ce = new CustomEvent(m_handler,COOKIE_ERROR);
      m_handler.setProperty(COOKIE_ERROR_MESSAGE,message);
      dispatchCustomEvent(ce);
      }
      catch (Exception e)
      {
        e.printStackTrace();
      }
  }

  /**
   * Convenience method to display an message out to the console.  Examines the
   * m_Debug and m_DebugALL settings to see if it should output the message.  Outputs message in
   * format <classname : msg >
   *
   * @param the message to be output
   */
  public void log(String msg)
  {
    if(m_debug)
        System.out.println(CLASSNAME + ": " + msg);
  }
}